@@ -1,5 +1,7 @@
+import cStringIO as StringIO
 import json
 import os
+import datetime
 from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
@@ -11,8 +13,8 @@ import beanstalk
 from beanstalk.tasks.utils import (set_verbose_level, load_role_settings as load_beanstalk_settings,
                                    load_web_servers, run_hooked_actions)
 from beanstalk.decorators import beanstalk_role
-from beanstalk.titles import section_title, tool_title
-from beanstalk.validator import validate_web_server, validate_file_existence
+from beanstalk.titles import section_title, tool_title, separator
+from beanstalk.validator import validate_web_server, validate_file_existence, validate_project_type, supported_projects
 
 
 @task
@@ -23,7 +25,7 @@ def web_server(*servers):
     hosts = []
     for server in servers:
         try:
-            hosts += map(lambda x: '%s' % x, list(IPNetwork(server)))
+            hosts += map(lambda x: '{0}'.format(x), list(IPNetwork(server)))
         except (ValueError, AddrFormatError):
             hosts.append(server)
 
@@ -41,56 +43,94 @@ def web_server(*servers):
 
 
 @task
+@beanstalk_role('app')
 def plant(VERBOSE=0):
     """plant beanstalk-stack for project in current work directory
     """
-    print magenta('Beanstalk-Stack app plant tool')
-    print '=-' * 40
-    # Get values
-    setting_values = [
-        (
-            'PROJECT_SOURCE_ROOT',
-            prompt('Project Source Root?', default=os.getcwd()),
-            os.getcwd(),
-            'str',
-        ),
-        (
-            'PROJECT_NAME',
-            prompt('Project Name?', default=os.path.split(os.getcwd())[-1]),
-            os.path.split(os.getcwd())[-1],
-            'str',
-        ),
-        (
-            'GIT_IGNORE_PATH',
-            prompt('Where\'s git ignore file?', default='.gitignore', validate=validate_file_existence),
-            '.gitignore',
-            'str',
-        ),
-        (
-            'WEB_SERVERS',
-            prompt('Where to deploy? (Python list/tuple with server address)', validate=validate_web_server),
-            None,
-            'obj',
-        ),
-    ]
-    print ''
-
     set_verbose_level(VERBOSE)
-
     # Create beanstalk_stack home
     beanstalk_local_base_path = os.path.normpath(os.path.join(os.getcwd(), BEANSTALK_LOCAL_BASE_PATH))
+    app_settings_path = os.path.normpath(os.path.join(beanstalk_local_base_path, 'beanstalk_app_settings.py'))
+
     if not os.path.exists(beanstalk_local_base_path):
-        print cyan('Make beanstalk_stack local base: ./%s' % os.path.relpath(beanstalk_local_base_path))
-        local('mkdir -p %s' % beanstalk_local_base_path)
+        print cyan('Make beanstalk_stack local base: ./{path}'.format(path=os.path.relpath(beanstalk_local_base_path)))
+        local('mkdir -p {path}'.format(path=beanstalk_local_base_path))
+
+    print magenta('Beanstalk-Stack app plant tool')
+    print separator('=-')
+
+    result = {}
+
+    def collector(setting_sets):
+        for setting_item in setting_sets:
+            if setting_item is None:
+                print separator()
+                continue
+            key, question, validator, value_type = setting_item
+            default_value = beanstalk_default_settings[key] or ''
+            value = prompt(question, default=default_value, validate=validator)
+            if value != default_value:
+                result[key] = (value, value_type)
+
+        for key, (value, value_type) in result.items():
+            beanstalk_default_settings[key] = value
+
+    # Collect
+    beanstalk_default_settings = load_beanstalk_settings(default_only=True)
+    setting_set1 = [
+        ('PROJECT_SOURCE_ROOT', 'Project Source Root?', None, 'str'),
+        ('PROJECT_NAME', 'Project Name?', None, 'str'),
+        ('PROJECT_VENV_PATH', 'Where\'s your virtualenv?', validate_file_existence, 'obj'),
+        ('GIT_IGNORE_PATH', 'Where\'s git ignore file?', validate_file_existence, 'str'),
+        None,
+        ('WEB_SERVERS', 'Where to deploy? (Python list/tuple with server address)', validate_web_server, 'obj'),
+        None,
+        ('PROJECT_TYPE',
+         'Which project type you are? (\'plain\' for project not in ({projects}))'.format(
+             projects=', '.join(supported_projects)),
+         validate_project_type, 'str'),
+    ]
+    collector(setting_set1)
+
+    # Project type
+    if ('PROJECT_TYPE' in result and result['PROJECT_TYPE'] == 'django') or ('PROJECT_TYPE' not in result):
+        setting_set2 = [
+            ('DJANGO_PROJECT_SETTINGS', 'Where\'s your django project settings? (normal one)',
+             validate_file_existence, 'str'),
+            ('DJANGO_PROJECT_SETTINGS_DEV', 'Where\'s your django project settings for development?',
+             validate_file_existence, 'str'),
+            ('DJANGO_PROJECT_SETTINGS_PROD', 'Where\'s your django project settings for production?',
+             validate_file_existence, 'str'),
+        ]
+        collector(setting_set2)
+
+    print ''
 
     # Generate beanstalk_settings
-    app_settings_path = os.path.normpath(os.path.join(beanstalk_local_base_path, 'beanstalk_app_settings.py'))
-    if not os.path.exists(app_settings_path):
-        print cyan('Generate beanstalk_stack app settings')
-        with open(os.path.join(beanstalk_local_base_path, 'beanstalk_app_settings.py'), 'w') as f:
-            for key, value, default, type in setting_values:
-                if value != default:
-                    f.write('%s = %s\n' % (key, value if type == 'obj' else '\'%s\'' % value))
+    print cyan('Generate beanstalk_stack app settings')
+    sorted_result = []
+    for key, (value, value_type) in result.items():
+        sorted_result.append((key, (value, value_type)))
+    sorted_result.sort(cmp=lambda x, y: cmp(x[0], y[0]))
+
+    print 'Genreated setting file:'
+    print separator('=')
+    output = StringIO.StringIO()
+    for key, (value, value_type) in sorted_result:
+        value = value if value_type == 'obj' else '\'{0}\''.format(value)
+        output.write('{0} = {1}\n'.format(key, value))
+    final_settings = output.getvalue()
+    print final_settings
+    output.close()
+    print separator('=')
+
+    write_to_file = confirm('Write to {path}'.format(path=app_settings_path), default=True)
+    if write_to_file:
+        if os.path.exists(app_settings_path):
+            local('mv {path} {path}.bak.{ts}'.format(
+                path=app_settings_path, ts=datetime.datetime.now().strftime('%s-%f')))
+        with open(app_settings_path, 'w') as f:
+            f.write(final_settings)
 
     print green('Planted. Grow up now!')
 
@@ -102,6 +142,7 @@ def deploy(settings_path=None, **settings_patches):
     """
     # Set variables
     env.work_directory = os.getcwd()
+    env.in_remote = False
 
     # Load beanstalk settings
     settings_patches = eval_kwargs(settings_patches)
@@ -119,16 +160,17 @@ def deploy(settings_path=None, **settings_patches):
 
     # Start
     print tool_title('Beanstalk-Stack app deploy tool')
-    print '=-' * 40
+    print separator('=-')
     print 'Project Name   : ' + green(project_name)
     print 'Source root    : ' + green(project_source_root)
-    print 'Target servers : ' + yellow('%s' % env.roledefs['web_servers'])
-    print '-' * 80
+    print 'Target servers : ' + yellow('{0!s}'.format(env.roledefs['web_servers']))
+    print separator('-')
 
     # Start to work in room!
-    with temporary_directory(identifier=beanstalk_identifier, prefix='deploy~%s~' % project_name) as tmp_directory:
+    tmp_dir_prefix = 'deploy~{0}~'.format(project_name)
+    with temporary_directory(identifier=beanstalk_identifier, prefix=tmp_dir_prefix) as tmp_directory:
         print section_title('Copy source root to temp directory')
-        puts('tmp dir: %s' % tmp_directory)
+        puts('tmp dir: {path}'.format(path=tmp_directory))
         env.temp_directory = tmp_directory
 
         # Find the exclude file (gitignore)
@@ -140,21 +182,23 @@ def deploy(settings_path=None, **settings_patches):
             if not gitignore_at_source_root:
                 local(rsync(gitignore_path, os.path.join(tmp_directory, '.gitignore')))
         else:
-            abort('Cannot find gitignore file at %s' % gitignore_path)
+            abort('Cannot find gitignore file at {path}'.format(path=gitignore_path))
 
         # Copy it
-        rsync_argument = '-rlpctzD --exclude .git --exclude-from \'%s\'' % os.path.relpath(gitignore_path)
+        rsync_argument = '-rlpctzD --exclude .git --exclude-from \'{path}\''.format(
+            path=os.path.relpath(gitignore_path))
         local(rsync(project_source_root, tmp_directory, rsync_argument=rsync_argument, expand_to_destination=True))
 
         # Go to the tmp work dir
         with lcd(tmp_directory):
-            puts(section_title('Switch to %s' % tmp_directory))
+            puts(section_title('Switch to {path}'.format(path=tmp_directory)))
 
             # We wanna preserve pyc/pyo files in our remote
             # Remove *.pyc, *.pyo, *.py[co] in ignore patterns and append "logs/" in the ignore file
             local(
-                'sed -i.bak -e \'/^\*\.py[c|o]$/d;/^\*\.py\[co\]$/d;$a\\\nlogs\/;$a\\\nvenv\/\' %s && rm -rf %s.bak' %
-                ('.gitignore', '.gitignore')
+                'sed -i.bak -e '
+                '\'/^\*\.py[c|o]$/d;/^\*\.py\[co\]$/d;$a\\\nlogs\/;$a\\\nvenv\/\''
+                ' {path} && rm -rf {path}.bak'.format(path='.gitignore')
             )
 
             # Check source code first
@@ -175,9 +219,12 @@ def deploy(settings_path=None, **settings_patches):
                 print ''
 
             # Work on remote
-            print '+' * 80
+            print separator('+')
             with settings(hide('running')):
+                env.in_remote = True
                 execute(deploy_remote_core)
+                env.in_remote = False
+            print separator('+')
 
             # Call post-deploy script
             if run_hooked_actions('POST_DEPLOY_ACTIONS'):
@@ -192,7 +239,7 @@ def deploy_remote_core():
     """ The part to run on each server
     """
     env.host_name = host_name = env.host_string.split('@')[-1]
-    print section_title('Deploy to %s' % white(host_name))
+    print section_title('Deploy to {host}'.format(host=white(host_name)))
 
     # Find Jack first
     with settings(hide('everything'), warn_only=True):
@@ -203,30 +250,36 @@ def deploy_remote_core():
             )
         remote_bs_version = json.loads(run('bsjack info.version:USE_JSON=True --hide=status'))
         if beanstalk.VERSION != tuple(remote_bs_version['list']):
-            abort(red('Hey! Jack at the remote is not the same as me. He\'s %s. But I\'m %s.') %
-                  (remote_bs_version['string'], beanstalk.__version__))
+            abort(red('Hey! Jack at the remote is not the same as me. He\'s {0}. But I\'m {1}.').format(
+                remote_bs_version['string'], beanstalk.__version__))
 
     # Get/Set variables
     beanstalk_settings = env.beanstalk_settings
     project_name = beanstalk_settings['PROJECT_NAME']
     verbose_level = beanstalk_settings['VERBOSE']
 
-    # Create this app to server
-    print section_title('Find room for %s' % project_name)
-    run('bsjack server.create_app:%s,VERBOSE=%d --hide=status' % (project_name, beanstalk_settings['VERBOSE']))
-
     # Call pre-install script
     run_hooked_actions('PRE_INSTALL_ACTIONS')
 
+    # Create this app to server
+    print section_title('Find room for %s' % project_name)
+    run('bsjack server.create_app:{project_name},VERBOSE={verbose:d} --hide=status'.format(
+        project_name=project_name, verbose=beanstalk_settings['VERBOSE']))
+
     # Get app info first
     with hide('everything'):
-        app_info = json.loads(run('bsjack server.app_info:%s,VERBOSE=%d,USE_JSON=True --hide=status' %
-                                  (project_name, beanstalk_settings['VERBOSE'])))
+        app_info = json.loads(
+            run(
+                'bsjack server.app_info:{project_name},VERBOSE={verbose:d},USE_JSON=True --hide=status'.format(
+                    project_name=project_name, verbose=beanstalk_settings['VERBOSE'])))
     env.remote_app_info = app_info
     remote_base_path = app_info['path']['base']
     remote_source_path = app_info['path']['source']
 
-    print section_title('Send %s to remote: %s' % (project_name, white(remote_base_path)))
+    # Call pre-install script
+    run_hooked_actions('PRE_SEND_ACTIONS')
+
+    print section_title('Send {name} to remote: {path}'.format(name=project_name, path=white(remote_base_path)))
     # Send code
     rsync_argument = '-DLhrctz --delete --delete-excluded'
     rsync_argument += ' --exclude \'*.pyc\' --exclude \'*.pyo\' --exclude-from \'.gitignore\''
@@ -236,24 +289,27 @@ def deploy_remote_core():
         rsync_argument += ' -v'
     elif verbose_level == 3:
         rsync_argument += ' -Pv'
-    target_source_path = 'beanstalk@%s:%s' % (host_name, remote_source_path)
-    target_base_path = 'beanstalk@%s:%s' % (host_name, remote_base_path)
+    target_source_path = 'beanstalk@{0}:{1}'.format(host_name, remote_source_path)
+    target_base_path = 'beanstalk@{0}:{1}'.format(host_name, remote_base_path)
     local(rsync(env.temp_directory, target_source_path, rsync_argument=rsync_argument, expand_to_destination=True))
-    local(rsync('%s/.gitignore' % env.temp_directory, target_base_path))
+    local(rsync('{path}/.gitignore'.format(path=env.temp_directory), target_base_path))
 
     # Call post-install script
     run_hooked_actions('POST_SEND_ACTIONS')
 
     # Update this app in server
     print section_title('Install %s in remote' % project_name)
-    run('bsjack server.install_app:%s,VERBOSE=%d --hide=status' % (project_name, beanstalk_settings['VERBOSE']))
+    run('bsjack server.install_app:{name},VERBOSE={verbose:d} --hide=status'.format(
+        name=project_name, verbose=beanstalk_settings['VERBOSE']))
 
     # Call post-install script
     run_hooked_actions('POST_INSTALL_ACTIONS')
 
     # Commit state to git repo
     print section_title('Commit %s in remote' % project_name)
-    run('bsjack server.commit_app:%s,VERBOSE=%d --hide=status' % (project_name, beanstalk_settings['VERBOSE']))
-    run('bsjack server.reload_app:%s,VERBOSE=%d --hide=status' % (project_name, beanstalk_settings['VERBOSE']))
+    run('bsjack server.commit_app:{name},VERBOSE={verbose:d} --hide=status'.format(
+        name=project_name, verbose=beanstalk_settings['VERBOSE']))
+    run('bsjack server.reload_app:{name},VERBOSE={verbose:d} --hide=status'.format(
+        name=project_name, verbose=beanstalk_settings['VERBOSE']))
 
-    print ' . ' * 26
+    print separator(' . ')
