@@ -10,58 +10,12 @@ from ground_soil.fabric import eval_kwargs, virtual_env
 from ground_soil.filesystem import temporary_directory, render_file
 from beanstalk import IDENTIFIER, BEANSTALK_ROOT_PATH
 from beanstalk.decorators import beanstalk_role
+from beanstalk.paths.server import (project_base_path, project_source_path, project_logs_path, project_venv_path,
+                                    project_uwsgi_path, apache_conf_path, app_ports_path)
 from beanstalk.tasks.utils import load_role_settings as load_beanstalk_settings, set_verbose_level, run_hooked_actions
-from beanstalk.titles import tool_title, section_title
+from beanstalk.titles import tool_title, section_title, separator
 
 
-def project_base_path(project_name):
-    beanstalk_settings = env.get('beanstalk_settings', None)
-    if beanstalk_settings is None:
-        abort('Load beanstalk settings first')
-
-    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
-    project_base = os.path.abspath(os.path.join(beanstalk_stack_base, 'apps/', project_name))
-
-    return project_base
-
-
-def project_source_path(project_name):
-    project_base = project_base_path(project_name)
-    return os.path.join(project_base, 'source/')
-
-
-def project_logs_path(project_name):
-    project_base = project_base_path(project_name)
-    return os.path.join(project_base, 'logs/')
-
-
-def project_venv_path(project_name):
-    project_base = project_base_path(project_name)
-    return os.path.join(project_base, 'venv/')
-
-
-def project_uwsgi_path(project_name):
-    project_base = project_base_path(project_name)
-    return os.path.join(project_base, 'uwsgi.ini')
-
-
-def apache_conf_path():
-    beanstalk_settings = env.get('beanstalk_settings', None)
-    if beanstalk_settings is None:
-        abort('Load beanstalk settings first')
-
-    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
-    return os.path.join(beanstalk_stack_base, 'confs/', 'apache.conf')
-
-
-def app_ports_path():
-    beanstalk_settings = env.get('beanstalk_settings', None)
-    if beanstalk_settings is None:
-        abort('Load beanstalk settings first')
-
-    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
-    return os.path.join(beanstalk_stack_base, 'confs/', 'ports.json')
-
 # Tasks
 #-----------------------------------------------------------------------------------------------------------------------
 
@@ -80,6 +34,9 @@ def setup(settings_path=None, **settings_patches):
     beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
     set_verbose_level()
 
+    # Call pre-setup script
+    run_hooked_actions('PRE_SETUP_ACTIONS')
+
     # Install packages for building uwsgi
     print section_title('Install system packages')
     local('yum install -y python-virtualenv python-devel sqlite-devel libxml2-devel pcre-devel zeromq-devel'
@@ -187,8 +144,8 @@ def setup(settings_path=None, **settings_patches):
     local('/etc/init.d/beanstalk start')
     local('chkconfig beanstalk on')
 
-    # Call post-install script
-    run_hooked_actions('POST_INSTALLATION_ACTIONS')
+    # Call post-setup script
+    run_hooked_actions('POST_SETUP_ACTIONS')
 
 
 @task
@@ -198,6 +155,7 @@ def create_app(project_name, settings_path=None, **settings_patches):
     """
     settings_patches = eval_kwargs(settings_patches)
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    env.project_name = project_name
     set_verbose_level()
 
     # Folder paths
@@ -210,6 +168,9 @@ def create_app(project_name, settings_path=None, **settings_patches):
         print 'Nothing to do. App existed!'
         return
 
+    # Pre-create actions
+    run_hooked_actions('PRE_CREATE_ACTIONS')
+
     # Create folders
     print section_title('Create folders')
     local('mkdir -p %s' % project_base)
@@ -255,6 +216,7 @@ def create_app(project_name, settings_path=None, **settings_patches):
     if not port_available:
         abort('No available port for uwsgi')
     else:
+        env.uwsgi_port = uwsgi_port
         ports.append(uwsgi_port)
     # Save it
     with open(app_ports_path(), 'w') as f:
@@ -281,6 +243,9 @@ def create_app(project_name, settings_path=None, **settings_patches):
     port_file_path = os.path.join(project_base_path(project_name), '%d.port' % uwsgi_port)
     local('echo "%d" > %s' % (uwsgi_port, port_file_path))
 
+    # Post-create actions
+    run_hooked_actions('POST_CREATE_ACTIONS')
+
 
 @task
 @beanstalk_role('server')
@@ -290,6 +255,7 @@ def delete_app(project_name, settings_path=None, **settings_patches):
     settings_patches = eval_kwargs(settings_patches)
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
+    env.project_name = project_name
 
     # Folder paths
     project_base = project_base_path(project_name)
@@ -324,6 +290,9 @@ def install_app(project_name, settings_path=None, **settings_patches):
     settings_patches = eval_kwargs(settings_patches)
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
+    env.project_name = project_name
+
+    run_hooked_actions('PRE_INSTALL_ACTIONS')
 
     project_source = project_source_path(project_name)
     project_venv = project_venv_path(project_name)
@@ -367,6 +336,8 @@ def install_app(project_name, settings_path=None, **settings_patches):
     with open(uwsgi_path, 'wb') as f:
         config.write(f)
 
+    run_hooked_actions('POST_INSTALL_ACTIONS')
+
 
 @task
 @beanstalk_role('server')
@@ -376,10 +347,15 @@ def reload_app(project_name, settings_path=None, **settings_patches):
     settings_patches = eval_kwargs(settings_patches)
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
+    env.project_name = project_name
+
+    run_hooked_actions('PRE_RELOAD_ACTIONS')
 
     print section_title('Reload app: %s' % project_name)
     local('touch %s' % project_uwsgi_path(project_name))
 
+    run_hooked_actions('POST_RELOAD_ACTIONS')
+
 
 @task
 @beanstalk_role('server')
@@ -451,12 +427,12 @@ def app_info(project_name, settings_path=None, **settings_patches):
         return json_string
     else:
         print section_title('Beanstalk-stack app: %s' % project_name)
-        print '-=' * 40
+        print separator('-=')
         print 'Base   : %s' % project_base_path(project_name)
         print 'Source : %s' % project_source_path(project_name)
         print 'Logs   : %s' % project_logs_path(project_name)
         print 'venv   : %s' % project_venv_path(project_name)
-        print '-' * 40
+        print separator('-', 40)
         print 'uwsgi port : %s' % uwsgi_port
 
 
@@ -472,7 +448,7 @@ def list_apps(settings_path=None, **settings_patches):
     apps = os.listdir(os.path.join(beanstalk_settings['BEANSTALK_STACK_BASE'], 'apps/'))
 
     print section_title('Beanstalk-stack apps')
-    print '-=' * 40
+    print separator('-=')
     for app in apps:
         print app
 
@@ -484,8 +460,13 @@ def commit_app(project_name, settings_path=None, **settings_patches):
     """
     settings_patches = eval_kwargs(settings_patches)
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    env.project_name = project_name
     set_verbose_level()
 
+    run_hooked_actions('PRE_COMMIT_ACTION')
+
     project_base = project_base_path(project_name)
     with settings(lcd(project_base), hide('warnings'), warning=True):
         local('git add -A && git commit -m "state at %s"' % datetime.datetime.now().strftime('%s'))
+
+    run_hooked_actions('POST_COMMIT_ACTION')
