@@ -34,3 +34,11 @@ def validate_file_existence(raw_value):
         raise ValueError('There\'s no such file at this path.')
 
     return os.path.normpath(os.path.expanduser(raw_value))
+
+
+def validate_project_type(raw_value):
+    if raw_value.lower() in ('plain',) + supported_projects:
+        return raw_value.lower()
+    raise ValueError('\'{type}\' is not a supported project type. Use \'plain\' for other project'.format(
+        type=raw_value))
+supported_projects = ('django',)
